/*

  Program Forced_oscillator.ccp:
  solves the equation of motion for a forced and damped oscillator.
  Equation:

  d^2 x / dt^2 = -2 * gamma * dx / dt - w^0 x + F * cos( w * t )

 */

#include <iostream>
#include <fstream>
#include <cmath>
using namespace std;

#include "RK2.h"


// Global variables (parameters of the equation, to use in dydt!)

double gam, w0, F, w;


// Forward defined function dydt

void dydt( int, double, double *, double * );


/*

  Main program

*/

int main()
{

  cout << "Program to solve the equation of a forced and damped oscillator" << endl;
  cout << "through a second order Runge-Kutta time scheme." << endl;

  int Neq = 2;

  cout << "Value of dt, Tend, Tprint: ";
  double dt, Tend, Tprint;
  cin >> dt >> Tend >> Tprint;

  long Nend   = long( Tend / dt );
  long Nprint = long( Tprint / dt );

  cout << "Parameters gamma, w0, F, w in the equation: ";
  cin >> gam >> w0 >> F >> w;

  cout << "Initial conditions x0, v0: ";
  double y[Neq];
  cin >> y[0] >> y[1];

  RK2 sol = RK2( Neq, y );

  cout << "Output file name: ";
  char nfile[256];
  cin >> nfile;

  double t_n = 0.0;

  // Open the output file and writes the initial conditions

  ofstream fout;
  fout.open( nfile );
  fout.precision( 15 );
  fout << "#   t     x    v" << endl;
  fout << t_n;
  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
  fout << endl;

  // Time loop

  for ( long n = 1; n <= Nend; n++ )
    {

      // One step forward through the RK2 scheme

      sol.stepRK2( t_n, dt, dydt );
      t_n = n * dt;

      // Print each Tprint time intervals

      if ( ( n % Nprint ) == 0 )
	{
	  sol.getValue( y );
	  fout << t_n;
	  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
	  fout << endl;
	}

    }

  fout.close();

}


// Function dydt: computes the RHS of the system of equations

void dydt( int Neq, double t, double * y, double * dy )
{
  dy[0] = y[1];
  dy[1] = - gam * y[1] - w0 * w0 * y[0] + F * cos( w * t );
}
