/*

  Program Pendulum.ccp:
  integrate the motion equation for a pendulum.
  Equation:

  d^2 x / dt^2 = - w^2 sin(x)

 */

#include <iostream>
#include <fstream>
#include <cmath>
using namespace std;

#include "RK2.h"


// Global variables (parameters of the equation, to use in dydt!)

double w;


// Forward defined function dydt

void dydt( int, double, double *, double * );


/*

  Main program

*/

int main()
{

  cout << "Program to integrate the equation of a pendulum" << endl;
  cout << "through a second order Runge-Kutta time scheme." << endl;

  int Neq = 2;

  cout << "Value of dt, Tend, Tprint: ";
  double dt, Tend, Tprint;
  cin >> dt >> Tend >> Tprint;

  long Nend   = long( Tend / dt );
  long Nprint = long( Tprint / dt );

  cout << "Parameter w in the equation: ";
  cin >> w;

  cout << "Initial conditions x0, v0: ";
  double y[Neq];
  cin >> y[0] >> y[1];

  RK2 sol = RK2( Neq, y );

  cout << "Output file name: ";
  char nfile[256];
  cin >> nfile;

  double t_n = 0.0;

  // Physical invariant: Energy

  double E  = 0.5 * y[1] * y[1] + w * w * ( 1.0 - cos( y[0] ) );

  // Open the output file and writes the initial conditions

  ofstream fout;
  fout.open( nfile );
  fout.precision( 15 );
  fout << "#   t     x    v   E" << endl;
  fout << t_n;
  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
  fout << "   " << E << endl;

  // Time loop

  for ( long n = 1; n <= Nend; n++ )
    {

      // One step forward through the RK2 scheme

      sol.stepRK2( t_n, dt, dydt );
      t_n = n * dt;

      // Print each Tprint time intervals

      if ( ( n % Nprint ) == 0 )
	{
	  sol.getValue( y );
	  double E  = 0.5 * y[1] * y[1] + w * w * ( 1.0 - cos( y[0] ) );
	  fout << t_n;
	  for ( int i = 0; i < Neq; i++ ) fout << "   " << y[i];
	  fout << "   " << E << endl;
	}

    }

  fout.close();

}


// Function dydt: computes the RHS of the system of equations

void dydt( int Neq, double t, double * y, double * dy )
{
  dy[0] = y[1];
  dy[1] = - w * w * sin( y[0] );
}
