/*

  File RK2.ccp:

  class to solve a single step of a n-th order differential equation,
  with the second order Runge-Kutta time scheme. The RHSs of the
  corresponding system of equations are passed as a function among the
  arguments of the call to the solver.

*/


#include "RK2.h"


// Constructor of the class

RK2::RK2( int Neq, double * y0 )
{
  N = Neq;
  y = new double[ N ];
  dy = new double[ N ];
  ys = new double[ N ];
  setValue( y0 );
}


// Destructor of the class

RK2::~RK2( )
{
  delete [] y;
  delete [] dy;
  delete [] ys;
}


// Function setValue: set the initial conditions in the vector of the unknowns

void RK2::setValue( double * y0 )
{
  for ( int i = 0; i < N; i++ ) y[i] = y0[i];
}


// Function getValue: return the value of the unknowns at the current time

void RK2::getValue( double * sol )
{
  for ( int i = 0; i < N; i++ ) sol[i] = y[i];
}


// Function stepRK2: execute the single second order Runge-Kutta time step.
// Note: the function that computes the RHSs of the system of equations is passed as a pointer
// to the stepRK2 function.

//void RK2::stepRK2( double t, double dt, void * dydt( int, double, double *, double * ) )
void RK2::stepRK2( double t, double dt, void ( * dydt )( int, double, double *, double * ) )
{
  ( * dydt )( N, t, y, dy );
  for ( int i = 0; i < N; i++ ) ys[i] = y[i] + 0.5 * dt * dy[i];
  double ts = t + 0.5 * dt;
  ( * dydt )( N, ts, ys, dy );
  for ( int i = 0; i < N; i++ ) y[i] = y[i] + dt * dy[i];
}
